/** Implies something is to be done at this point in the code in the future. */
declare function todo(): never;
/** Implies this code should be unreachable.
 *  Optionally also type-checks the argument to verify this is the case.
 */
declare function unreachable(_unreachableTypeCheck?: never): never;
/** While 'todo()' implies something is intended to be done in the future
 *  at this point in the code, 'unimplemented()' implies that something
 *  is intentionally not implemented.
 */
declare function unimplemented(): never;

/** Copies text to the clipboard in browser environments. */
declare function copyText(text: string): Promise<boolean>;

declare function newId(): number;
declare function setUniqueNodeId(node: HTMLElement, prefix: string): HTMLElement;

/** Enumerates over an array, returning both the value and the index. */
declare function enumerate<T>(iterable: Iterable<T>): Iterable<[number, T]>;
/** Filters an object by a predicate. */
declare function filterObject<T extends Record<string, unknown>>(obj: T, predicate: (key: keyof T, value: T[keyof T]) => boolean): Partial<T>;
/** Maps an object. */
declare function mapObject<T, U>(obj: Record<string, T>, map: (key: string, value: T) => U): Record<string, U>;
/** Filters and maps an object at the same time. */
declare function filterMapObject<T, U>(obj: Record<string, T>, predicate: (key: string, value: T) => null | U): Record<string, U>;
/** Filter and map an array at the same time. */
declare function filterMap<T, V>(array: Array<T>, predicate: (item: T) => null | V): Array<V>;

/** Same as `Number.prototype.toFixed` but floors the stringed value. */
declare function toFixedFloor(num: number, digits: number): string;
/** Truncates the given number to `precision` significant digits.
 *
 *  This differs from `toPrecision` which rounds the output value.
 */
declare function truncateToSignificantDigits(num: number, precision: number): string;
/** Remove the `e+/-n` in a stringified number, converting it into a proper
 *  format.
 */
declare function removeExponentialNotation(num: string): string;
/**
 * Linearly interpolates between two values.
 * @param start - The starting value.
 * @param end - The ending value.
 * @param t - The interpolation value between 0 and 1.
 * @returns The interpolated value between start and end.
 */
declare function lerp(start: number, end: number, t: number): number;
/** Euler mod of two numbers. */
declare function mod(a: number, b: number): number;

declare function noop(): void;
declare function asyncnoop(): Promise<void>;

/** An implementation of Rust's `Option` type in TypeScript. */
declare class Option<T> {
    private value;
    constructor(value: ["some", T] | ["none"]);
    static fromNullable<T>(value: T | null): Option<T>;
    isSome(): boolean;
    isNone(): boolean;
    unwrap(): T;
    expect(message: string): T;
    /** Similar to Rust's `match`. */
    match<U>(matcher: {
        some: (value: T) => U;
        none: () => U;
    }): U;
}
declare function some<T>(value: T): Option<T>;
declare function none<T = never>(): Option<T>;

/** An implementation of Rust's `Result` type in TypeScript. */
declare class Result<T, E> {
    private value;
    constructor(value: ["ok", T] | ["err", E]);
    isOk(): boolean;
    isErr(): boolean;
    unwrap(): T;
    unwrapErr(): E;
    expect(message: string): T;
    expectErr(message: string): E;
    map<U>(f: (value: T) => U): Result<U, E>;
    mapErr<F>(f: (value: E) => F): Result<T, F>;
    /** Fixes the typings on the `Err` variant of a Result. Unsafe operation.
     *
     *  # Example
     *  ```
     *  function a(): Result<string, string> {
     *      return ok("good news!");
     *  }
     *
     *  function b(): Result<string, number> {
     *      const result = a();
     *      if (result.isErr()) {
     *          return err(10);
     *      } else {
     *          // Without fix: Type error: string is not assignable to number
     *          return result.fixErr();
     *      }
     *  }
     *  ```
     */
    fixErr<B>(): Result<T, B>;
    /** Fixes the typings on the `Ok` variant of a Result. Throws if this value
     *  is an `Ok`.
     *
     *  # Example
     *  ```
     *  function a(): Result<string, string> {
     *      return err("oh no!");
     *  }
     *
     *  function b(): Result<number, string> {
     *      const result = a();
     *      if (result.isErr()) {
     *          // Without fix: Type error: string is not assignable to number
     *          return result.fixOk();
     *      } else {
     *          return ok(5);
     *      }
     *  }
     *  ```
     */
    fixOk<A>(): Result<A, E>;
    /** Similar to Rust's `match`. */
    match<U>(matcher: {
        ok: (value: T) => U;
        err: (error: E) => U;
    }): U;
    /** Flattens a `Result<Result<T, E>, E>` into a `Result<T, E>`. */
    flatten: T extends Result<infer U, E> ? () => Result<U, E> : never;
    /** Returns `undefined` on error, else the inner value. */
    undefined(): T | undefined;
    /** Throws `E` or returns `T`. */
    throw(): T;
}
declare function ok<const T, E = never>(value: T): Result<T, E>;
declare function ok<T, E = never>(value: T): Result<T, E>;
declare function err<T = never, const E = unknown>(value: E): Result<T, E>;
declare function err<T = never, E = unknown>(value: E): Result<T, E>;

/** Safely parses an integer.
 *
 *  If the input is not an integer, it returns `null`.
 *  If the number is not safely parsed (i.e., is `NaN` or `Infinity`),
 *  it returns `null`.
 *  In the event of a decimal number, such as `1.5`, it returns `null`.
 */
declare function safeParseInt(x: unknown): number | null;
/** Safely parses a float.
 *
 *  If the input is not a number or parseable number, it returns `null`.
 *  If the number is not safely parsed (i.e., is `NaN` or `Infinity`),
 *  it returns `null`.
 */
declare function safeParseFloat(x: unknown): number | null;
/** Tries to convert a value into a string. */
declare function tryString(thing: unknown): string;

type KeyResult = {
    /** How important this key is relative to other keys. Higher = closer to top.
     *
     *  For example, one might want to prioritize the name of a thing over its description.
     *  Suppose the name has a bias of 3 and the description has a bias of 1,
     *  then even if the description was an exact match, if the name starts with
     *  the query, the name would be prioritized over the description. However,
     *  if it was a 'character appears in order' match, the description would be prioritized.
     */
    bias: number;
    /** The actual string to compare against. */
    value: string;
    /** The minimum (inclusive, unweighted) priority. If something is less than
     *  this, it is discarded.
     *
     *  Preferably use something from `SEARCH_PRIORITIES`.
     *
     *  @default 0
     */
    minPriority?: number;
};
declare function filterByString<T>(searchQuery: string, data: Array<T> | undefined, key: (item: T) => string | Array<KeyResult>, filter?: (item: T) => boolean): Array<T>;
declare const SEARCH_PRIORITIES: {
    /** The search query is an exact match to the string.
     *  E.g., `pi` matches '**Pi**'
     */
    readonly EXACT_MATCH: 6;
    /** The search query appears, in full, at the start of the string as the
     *  *full* word.
     *  E.g., `pi` matches '**Pi** Shop'
     */
    readonly STARTS_WITH_FULL_WORD: 5;
    /** The search query appears, in full, at the start of the string.
     *  E.g., `pi` matches '**Pi**zza Card'
     */
    readonly STARTS_WITH: 4;
    /** The search query appears, in full, after a space (thus denoting the
     *  beggining of a word.)
     *  E.g., `pi` matches 'Papa's **Pi**zzeria'
     */
    readonly STARTS_WITH_AFTER_SPACE: 3;
    /** The search query appears, in full, somewhere in the string.
     *  E.g., `pi` matches 'Pop**pi**ng In'
     */
    readonly INCLUDES: 2;
    /** The characters in the search query appear, in order, in the string.
     *  E.g., `pi` matches '**P**ay **i**t Now'
     */
    readonly CHARACTERS_IN_ORDER: 1;
    /** The search query does not appear in the string.
     *  E.g., `pi` does not match 'Glopsop's Mop Shop'
     */
    readonly NO_MATCH: 0;
};

type TimeFormat<Abbr extends string, Next extends string> = `${`${number}${Abbr}` | `${number}${Abbr} ` | ""}${Next}`;
type Time = TimeFormat<"y", TimeMonth>;
type TimeMonth = TimeFormat<"mo", TimeDay>;
type TimeDay = TimeFormat<"d", TimeHour>;
type TimeHour = TimeFormat<"h", TimeMinute>;
type TimeMinute = TimeFormat<"m", TimeSecond>;
type TimeSecond = TimeFormat<"s", TimeMillisecond>;
type TimeMillisecond = `${number}ms` | "";
type TimeParseable = Time | number;
/** Convert a string length of time into milliseconds.
 *
 *  The string can be in the format of `1y 2mo 3d 4h 5m 6s 7ms`.
 *  Any of the segments can be omitted.
 *  Whitespace between the segments is optional.
 *  The segments can *not* be in any order.
 *
 *  Examples:
 *  - `ms("6s749ms")` -> `6749`
 *  - `ms("6h")` -> `21600000`
 *  - `ms("8m -14s")` -> `466000`
 */
declare function ms(time: TimeParseable): number;
/** Formats a duration in milliseconds for `x:xx` or `x:xx:xx` format,
 *  depending on if hours need to be specified.
 */
declare function formatDuration(duration: TimeParseable): string;
/** Formats a date relative to the current time.
 *  Moderately opinionated about the format, but should be good for most cases.
 *
 *  Examples:
 * - `formatRelativeTime(new Date())` -> `Just now`
 * - `formatRelativeTime(new Date(Date.now() - ms("1d")))` -> `Yesterday`
 * - `formatRelativeTime(new Date(Date.now() - ms("1m 14s")))` -> `A minute ago`
 * - `formatRelativeTime(new Date(Date.now() - ms("6h 3m")))` -> `6 hours ago`
 * - `formatRelativeTime(new Date(Date.now() + ms("2s")))` -> `In a moment`
 * - `formatRelativeTime(new Date(Date.now() + ms("12d")))` -> `In 12 days`
 * - `formatRelativeTime(new Date(Date.now() + ms("45d")))` -> `6 Aug`
 * - `formatRelativeTime(new Date(Date.now() + ms("130d")))` -> `30 Oct 2024`
 */
declare function formatRelativeTime(date: Date, config?: {
    lowercase?: boolean;
    now?: Date;
}): string;
/** Calls `setTimeout` and returns a function that calls `clearTimeout` */
declare function createTimeout(callback: () => unknown, time: TimeParseable): () => void;

type RequiredKeys<T, K extends keyof T> = Omit<T, K> & Required<Pick<T, K>>;
type OptionalKeys<T, K extends keyof T> = Omit<T, K> & {
    [P in K]?: T[P] | undefined;
};
type ValueOf<T> = T[keyof T];
/** Create an array of the same length as `T` but with type `U`. */
type RepeatForLength<T extends Array<unknown>, U> = T extends [
    infer _,
    ...infer R
] ? [U, ...RepeatForLength<R, U>] : [];
/** Creates an array of `T` with length `L` */
type ArrayOfLength<T, L extends number> = L extends 0 ? [] : L extends 1 ? [T] : [T, ...ArrayOfLength<T, Decrement<L>>];
/** Subtracts 1. */
type Decrement<N extends number> = [never, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20][N];
/** Creates an array of `T` requiring at least one element in it. */
type NonEmptyArray<T> = [T, ...Array<T>];

export { type ArrayOfLength, type Decrement, type NonEmptyArray, Option, type OptionalKeys, type RepeatForLength, type RequiredKeys, Result, SEARCH_PRIORITIES, type TimeParseable, type ValueOf, asyncnoop, copyText, createTimeout, enumerate, err, filterByString, filterMap, filterMapObject, filterObject, formatDuration, formatRelativeTime, lerp, mapObject, mod, ms, newId, none, noop, ok, removeExponentialNotation, safeParseFloat, safeParseInt, setUniqueNodeId, some, toFixedFloor, todo, truncateToSignificantDigits, tryString, unimplemented, unreachable };
