type TimeFormat<Abbr extends string, Next extends string> = `${`${number}${Abbr}` | `${number}${Abbr} ` | ""}${Next}`;
type Time = TimeFormat<"y", TimeMonth>;
type TimeMonth = TimeFormat<"mo", TimeDay>;
type TimeDay = TimeFormat<"d", TimeHour>;
type TimeHour = TimeFormat<"h", TimeMinute>;
type TimeMinute = TimeFormat<"m", TimeSecond>;
type TimeSecond = TimeFormat<"s", TimeMillisecond>;
type TimeMillisecond = `${number}ms` | "";
export type TimeParseable = Time | number;
/** Convert a string length of time into milliseconds.
 *
 *  The string can be in the format of `1y 2mo 3d 4h 5m 6s 7ms`.
 *  Any of the segments can be omitted.
 *  Whitespace between the segments is optional.
 *  The segments can *not* be in any order.
 *
 *  Examples:
 *  - `ms("6s749ms")` -> `6749`
 *  - `ms("6h")` -> `21600000`
 *  - `ms("8m -14s")` -> `466000`
 */
export declare function ms(time: TimeParseable): number;
/** Formats a duration in milliseconds for `x:xx` or `x:xx:xx` format,
 *  depending on if hours need to be specified.
 */
export declare function formatDuration(duration: TimeParseable): string;
/** Formats a date relative to the current time.
 *  Moderately opinionated about the format, but should be good for most cases.
 *
 *  Examples:
 * - `formatRelativeTime(new Date())` -> `Just now`
 * - `formatRelativeTime(new Date(Date.now() - ms("1d")))` -> `Yesterday`
 * - `formatRelativeTime(new Date(Date.now() - ms("1m 14s")))` -> `A minute ago`
 * - `formatRelativeTime(new Date(Date.now() - ms("6h 3m")))` -> `6 hours ago`
 * - `formatRelativeTime(new Date(Date.now() + ms("2s")))` -> `In a moment`
 * - `formatRelativeTime(new Date(Date.now() + ms("12d")))` -> `In 12 days`
 * - `formatRelativeTime(new Date(Date.now() + ms("45d")))` -> `6 Aug`
 * - `formatRelativeTime(new Date(Date.now() + ms("130d")))` -> `30 Oct 2024`
 */
export declare function formatRelativeTime(date: Date, config?: {
    lowercase?: boolean;
    now?: Date;
}): string;
/** Calls `setTimeout` and returns a function that calls `clearTimeout` */
export declare function createTimeout(callback: () => unknown, time: TimeParseable): () => void;
export {};
